/*
    This file is part of hat.

    hat is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    hat is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with hat.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <math.h>
#include "amoeba.h"

struct Data {
        double *y;         /* Data to be fitted */
        int turns;         /* Number of turns */
	double omega;
	double phi;
};

double
f(double *x, void *data)
{
        double *y;
        double aux;
        double omega, phi, amp, tau;
	double residual;
        int turns, t;

        turns = ((struct Data *)data)->turns;
        y = ((struct Data *)data)->y;
	omega = ((struct Data *)data)->omega;
	phi = ((struct Data *)data)->phi;

	amp = x[0];
	tau = x[1];

	residual = 0;
        for (t = 0; t < turns; t++) {
                aux = amp * cos(omega * (double)t + phi) * exp((double)t * tau) - y[t];
                residual += aux * aux;
  	}

        return residual;
}

double
fit_damping(double *buf, int turns, double freq, double phase)
{

        struct Data data;
	double simplex[3][2] = {{0., 0.}, {1., 0.}, {0., -1./100.}};
	int ilo;       

	data.turns = turns;
        data.y = buf;
	data.omega = freq * 2. * M_PI;
	data.phi = phase * 2. * M_PI;

	ilo = amoeba(simplex, 1e-6, f, (void *)&data);

	return -1. / simplex[ilo][1];
}

