/*
    This file is part of hat.

    hat is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    hat is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with hat.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <complex.h>
#include <math.h>

double complex
ortho_projection(double *buf, int count, double q)
{
        int i;
        double x;
	double complex z;

        q *= -2. * M_PI;

	z = 0;
        for (i = 0; i < count; i++) {
                x = buf[i];
                x *= 1. - cos(2. * M_PI * (double)i / (double)count);
		z += x * cexp(I * (double complex)q * (double complex)i);
        }

        z *= 2. / (double)count;

	return z;
}

double complex
ortho_projection_damped(double *buf, int count, double q, double damping)
{
        int i;
        double x;
	double complex z;

	damping = -1. * damping;
        q *= -2. * M_PI;

	z = 0;
        for (i = 0; i < count; i++) {
                x = buf[i];
		x *= exp((double)i / damping);
		x *= 1. - cos(2. * M_PI * (double)i / (double)count);
		z += x * cexp(I * (double complex)q * (double complex)i);
        }
	z *= 2. / (double)count;

	return z;
}

void
undamp(double *data, int turns, double k)
{
        int t;

        for (t = 0; t < turns; t++) {
                data[t] *= exp(-k * t);
        }
}

void
purge_line(double *data, int count, double q,
           double complex z, double damping)
{
        int i;
        double aux;

	q *= 2. * M_PI;

        for (i = 0; i < count; i++) {
                aux  = creal(z * cexp(I * q * (double)i));
                aux *= exp((double)i / damping);
                data[i] -= aux;
        }
}

